#include <xc.h>

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = ON        // Internal/External Switch Over (Internal External Switch Over mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PPS1WAY = ON     // Peripheral Pin Select one-way control (The PPSLOCK bit cannot be cleared once it is set by software)
#pragma config PLLEN = ON       // PLL Enable Bit (4x PLL is enabled when software sets the SPLLEN bit)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = ON         // Low-Voltage Programming Enable (Low-voltage programming enabled)

#define _XTAL_FREQ 32000000      // 内部クロックを32MHzで使用する場合の定義です。

// 正弦波データ配列（例として16点で1周期）
const unsigned char sine_wave[16] = {128, 176, 218, 245, 255, 245, 218, 176, 128, 79, 37, 10, 0, 10, 37, 79};
unsigned char i = 0; // ループカウンタとして割り込み内で使用

void __interrupt() isr(void) {
    if (TMR1IF) { // タイマ1の割り込みフラグをチェック
        TMR1IF = 0;  // タイマ1の割り込みフラグをクリア

        // AD変換開始
        ADCON0bits.GO_nDONE = 1;
        // AD変換完了待ち
        while(ADCON0bits.GO_nDONE);

        // AD変換結果に基づいて割り込み周期を設定
        TMR1 = 65535 - (ADRES>>2);

        DAC1CON1 = sine_wave[i];  // 正弦波の値をDACに設定
        i++;                     // インデックスをインクリメント
        if (i >= 16) i = 0;      // インデックスが配列のサイズを超えたら0にリセット
    }
}

void main(void) {
    // OSCコントロール
    OSCCON = 0xF0;  // 内部オシレータを32MHzに設定し、PLLENをONにしてPLLを有効にします。

    // DAC設定
    DAC1CON0 = 0x90; // DACを有効にし、正のリファレンスをVDDに、出力はオペアンプに接続
    DAC1CON1 = 0;    // DACの出力値を0に初期設定

    // オペアンプ設定
    OPA1CON = 0xD2;  // OPA1をUnity Gainモードで有効化, +入力にDAC outputを接続

        // オペアンプ2設定
    OPA2CON = 0xC0;  // OPA2をHigh GBWPモードで起動し、非反転入力をOPA2IN+ピンに、反転入力をOPA2IN-ピンに接続

    
    // ADC設定
    ANSELA = 0x10;   // RA4ピンをアナログ入力に設定
    ADCON0 = 0x0D;   // ADCを有効にし、チャンネルをAN3(RA4)に設定、ADONビットを設定
    ADCON1 = 0x20;   // 左詰め結果を選択し、FOSC/32の速度で変換

    // タイマ1設定
    T1CON = 0x01; // タイマ1を1:1プリスケーラで起動
    TMR1 = 65535 - ADRES>>2; // 初期値設定、割り込みが発生するまでの時間
    TMR1IE = 1;   // タイマ1割り込みを有効化
    PEIE = 1;     // 周辺装置割り込みを有効化
    GIE = 1;      // グローバル割り込みを有効化

    while (1) {
        // メインループでは何もしない
    }
}
